@@ -1,6 +1,6 @@
 /*
  * hostapd / Configuration file
- * Copyright (c) 2003-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2003-2006, Jouni Malinen <j@w1.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -451,7 +451,7 @@ int hostapd_setup_wpa_psk(struct hostapd_bss_config *conf)
 		memset(ssid->wpa_passphrase, 0,
 		       strlen(ssid->wpa_passphrase));
 		free(ssid->wpa_passphrase);
-		ssid->wpa_passphrase = 0;
+		ssid->wpa_passphrase = NULL;
 	}
 
 	if (ssid->wpa_psk_file) {
