@@ -2,7 +2,7 @@
  * hostapd / Driver interaction with MADWIFI 802.11 driver
  * Copyright (c) 2004, Sam Leffler <sam@errno.com>
  * Copyright (c) 2004, Video54 Technologies
- * Copyright (c) 2005-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2005-2006, Jouni Malinen <j@w1.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -440,8 +440,9 @@ madwifi_del_key(void *priv, const u8 *addr, int key_idx)
 
 	ret = set80211priv(priv, IEEE80211_IOCTL_DELKEY, &wk, sizeof(wk));
 	if (ret < 0) {
-		wpa_printf(MSG_DEBUG, "%s: Failed to delete key (addr " MACSTR
-			   " key_idx %d)", __func__, MAC2STR(addr), key_idx);
+		wpa_printf(MSG_DEBUG, "%s: Failed to delete key (addr %s"
+			   " key_idx %d)", __func__, ether_sprintf(addr),
+			   key_idx);
 	}
 
 	return ret;
@@ -499,10 +500,10 @@ madwifi_set_key(const char *ifname, void *priv, const char *alg,
 
 	ret = set80211priv(priv, IEEE80211_IOCTL_SETKEY, &wk, sizeof(wk));
 	if (ret < 0) {
-		wpa_printf(MSG_DEBUG, "%s: Failed to set key (addr " MACSTR
+		wpa_printf(MSG_DEBUG, "%s: Failed to set key (addr %s"
 			   " key_idx %d alg '%s' key_len %lu txkey %d)",
-			   __func__, MAC2STR(wk.ik_macaddr), key_idx, alg,
-			   (unsigned long) key_len, txkey);
+			   __func__, ether_sprintf(wk.ik_macaddr), key_idx,
+			   alg, (unsigned long) key_len, txkey);
 	}
 
 	return ret;
