@@ -1,6 +1,6 @@
 /*
  * hostapd / EAP-TTLS (draft-ietf-pppext-eap-ttls-05.txt)
- * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2004-2007, Jouni Malinen <j@w1.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -651,12 +651,11 @@ static Boolean eap_ttls_check(struct eap_sm *sm, void *priv,
 {
 	struct eap_hdr *resp;
 	u8 *pos;
-	size_t len;
 
 	resp = (struct eap_hdr *) respData;
 	pos = (u8 *) (resp + 1);
 	if (respDataLen < sizeof(*resp) + 2 || *pos != EAP_TYPE_TTLS ||
-	    (len = ntohs(resp->length)) > respDataLen) {
+	    (ntohs(resp->length)) > respDataLen) {
 		wpa_printf(MSG_INFO, "EAP-TTLS: Invalid frame");
 		return TRUE;
 	}
@@ -865,7 +864,7 @@ static void eap_ttls_process_phase2_mschapv2(struct eap_sm *sm,
 					     size_t challenge_len,
 					     u8 *response, size_t response_len)
 {
-	u8 *chal, *username, nt_response[24], *pos, *rx_resp, *peer_challenge,
+	u8 *chal, *username, nt_response[24], *rx_resp, *peer_challenge,
 		*auth_challenge;
 	size_t username_len, i;
 
@@ -894,7 +893,6 @@ static void eap_ttls_process_phase2_mschapv2(struct eap_sm *sm,
 	 * (if present). */
 	username = sm->identity;
 	username_len = sm->identity_len;
-	pos = username;
 	for (i = 0; i < username_len; i++) {
 		if (username[i] == '\\') {
 			username_len -= i + 1;
