@@ -1,6 +1,6 @@
 /*
  * hostapd / Initialization and configuration
- * Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2002-2007, Jouni Malinen <j@w1.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -680,6 +680,9 @@ static int hostapd_setup_encryption(char *iface, struct hostapd_data *hapd)
 			printf("Could not set WEP encryption.\n");
 			return -1;
 		}
+		if (hapd->conf->ssid.wep.key[i] &&
+		    i == hapd->conf->ssid.wep.idx)
+			hostapd_set_privacy(hapd, 1);
 	}
 
 	return 0;
@@ -1219,6 +1222,7 @@ static int setup_interface2(struct hostapd_iface *iface)
 	u8 *prev_addr;
 
 	hostapd_flush_old_stations(hapd);
+	hostapd_set_privacy(hapd, 0);
 
 	if (hapd->iconf->channel) {
 		freq = hostapd_hw_get_freq(hapd, hapd->iconf->channel);
@@ -1572,7 +1576,7 @@ static void show_version(void)
 		"hostapd v" VERSION_STR "\n"
 		"User space daemon for IEEE 802.11 AP management,\n"
 		"IEEE 802.1X/WPA/WPA2/EAP/RADIUS Authenticator\n"
-		"Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi> "
+		"Copyright (c) 2002-2007, Jouni Malinen <j@w1.fi> "
 		"and contributors\n");
 }
 
